use std::{io::Read, path::PathBuf};

use based::request::{assets::DataResponse, RequestContext, StringResponse};
use maud::html;
use rocket::{get, State};

pub mod component;
use component::*;

use crate::archive::WebsiteArchive;

/// Get the favicon of a domain
#[get("/favicon/<domain>")]
pub async fn favicon_route(domain: &str) -> Option<DataResponse> {
    let mut buf = Vec::new();
    std::fs::File::open(std::path::Path::new("./favicon").join(domain))
        .ok()?
        .read_to_end(&mut buf)
        .ok()?;

    Some(DataResponse::new(
        buf,
        "image/x-icon".to_string(),
        Some(60 * 60 * 24 * 5),
    ))
}

/// Websites Overview
#[get("/")]
pub async fn index(ctx: RequestContext, arc: &State<WebsiteArchive>) -> StringResponse {
    let websites = arc.domains();

    let content = html! {
        div class="container mx-auto p-4" {
            h1 class="text-5xl font-bold text-center mb-10" { "Websites" };
            div class="grid grid-cols-2 sm:grid-cols-3 lg:grid-cols-5 xl:grid-cols-6 2xl:grid-cols-8 gap-6" {

                @for site in websites {
                    a href=(format!("/d/{site}")) class="bg-neutral-900 shadow-md rounded-lg hover:bg-neutral-800 bg-gray-1 hover:cursor-pointer transition-all duration-300 flex flex-col items-center justify-center aspect-square max-w-60" {
                        div class="bg-blue-500 text-white rounded-full p-4" {
                            img class="h-8 w-8" src=(format!("/favicon/{site}")) {};
                        };
                        p class="mt-4 text-base font-medium" { (site) };
                    };
                };
            };
        }
    };

    render_page(content, ctx).await
}

/// Overview on <domain> / <path>
#[get("/d/<domain>/<paths..>")]
pub async fn domain_info_route(
    ctx: RequestContext,
    domain: &str,
    paths: PathBuf,
    arc: &State<WebsiteArchive>,
) -> StringResponse {
    let domain = arc.get_domain(domain);
    let document = domain.path(paths.to_str().unwrap());
    let versions: Vec<String> = document
        .versions()
        .into_iter()
        .map(|x| {
            x.trim_start_matches("index_")
                .trim_end_matches(".html")
                .to_string()
        })
        .collect();
    let (path_entries, is_doc) = domain.paths(paths.to_str().unwrap());
    let path_seperations: Vec<&str> = paths.to_str().unwrap().split('/').collect();

    // TODO : Show domains beeing linked on the page

    let content = html! {
        h2 class="text-xl font-bold mb-4 flex items-center" {
            img class="p-2" src=(format!("/favicon/{}", &domain.name)) {};
            a href=(format!("/d/{}", &domain.name)) { (domain.name) };
            (slash_seperator())
            (gen_path_header(path_seperations, &domain.name))
        };

        @if !versions.is_empty() {
            div class="max-w-md mx-auto p-4 bg-neutral-900 rounded-lg shadow-md mb-8" {
                h3 class="font-bold mb-2" { "Saved Versions:" };
                @for (index, ver) in versions.into_iter().enumerate() {
                    a href=(format!("/s/{}/{}?time={}", &domain.name, paths.to_str().unwrap(), ver)) class="text-xs text-blue-400" {

                        @if index == 0 {
                        (format!("Latest ({ver})"))
                        } @else {
                            (ver)
                        }
                    };
                }
            };
        }

        @if path_entries.is_empty() && !is_doc {
            div class="max-w-md mx-auto p-4 bg-neutral-900 rounded-lg shadow-md" {
                p class="p-4 text-xl" { "Nothing found" };
            }
        }

        @if !path_entries.is_empty() {
        div class="max-w-md mx-auto p-4 bg-neutral-900 rounded-lg shadow-md" {
            h3 class="font-bold mb-2" { "Paths:" };
            ul class="space-y-2" {
                @for path in path_entries {
                    a href=(path.url()) class="flex items-center gap-2 p-3 border bg-neutral-800 rounded hover:shadow-lg transition" {
                        (arrow_icon("blue"))

                        span class="font-medium" { (path.path()) };
                    };
                };
            };
        };
    };
    };

    render_page(content, ctx).await
}

/// Return archived version of `domain` / `path` at `time`
#[get("/s/<domain>/<path..>?<time>")]
pub async fn render_website(
    domain: &str,
    path: PathBuf,
    time: Option<&str>,
    arc: &State<WebsiteArchive>,
) -> Option<DataResponse> {
    let document = arc.get_domain(domain).path(path.to_str().unwrap());

    let content = document
        .render_local(time.map(|time| time.to_string()))
        .await;

    if let Some(content) = content {
        return Some(DataResponse::new(
            content.as_bytes().to_vec(),
            "text/html".to_string(),
            Some(60 * 60 * 24),
        ));
    } else if std::env::var("DOWNLOAD_ON_DEMAND")
        .unwrap_or("false".to_string())
        .as_str()
        == "true"
        && time.is_none()
    {
        arc.archive_url(&format!("https://{domain}/{}", path.to_str().unwrap()))
            .await;

        let content = document.render_local(None).await?;

        return Some(DataResponse::new(
            content.as_bytes().to_vec(),
            "text/html".to_string(),
            Some(60 * 60 * 24),
        ));
    }

    None
}
